@@ -6,18 +6,13 @@ use base 'MP3::Napster::IOEvent';
 use Carp 'croak';
 use constant MAX_BUFFER => 5120;
 
-use overload
-  '""' => 'asString',
-  fallback => 1;
-
 sub config {
   my $self = shift;
   my $args = shift;
   my $r = $args->{request};
   $self->request($r);
   $r->io_object($self);
-  $self->{prior} = '00';
-  $r->start;
+  $self->{prior} = [0,0];
 }
 
 # return the transfer request object
@@ -30,18 +25,18 @@ sub request {
 
 sub do_close {
   my $self = shift;
-  warn "MP3::Napster::Transfer::do_close() @_" if $self->eventloop->debug;
   $self->SUPER::do_close(@_);
   if (my $r = $self->request) {
     $r->io_object(undef);
     $r->done(1);
   }
+  $self->request(undef);
 }
 
 # modify can_read() to be true when the outbuffer is less than MAX_BUFFER
 sub can_read {
   my $self = shift;
-  $self->SUPER::can_read and length $self->outbuffer <= MAX_BUFFER;
+  $self->SUPER::can_read && length($self->outbuffer) <= MAX_BUFFER;
 }
 
 # modify incoming_data so that it is immediately written out
@@ -60,14 +55,10 @@ sub outgoing_data {
   $request->increment($bytes);
 }
 
-sub asString {
-  my $request = shift->request;
-  "IO object for $request";
-}
-
 sub DESTROY {
   my $self = shift;
-  warn "destroying $self" if $self->eventloop && $self->eventloop->debug;
+  warn "DESTROY $self" if $self->eventloop 
+    && $self->eventloop->debug;
   if (my $r = $self->request) {
     $self->request(undef);
   }
